﻿;==========================================================================================================================================================
; Encode a serial string of data
;==========================================================================================================================================================

Procedure.s EncodeSerialStream(SerialString.s)
  
  ;------------------------------------------------------------------------------------------------
  ;
  ;------------------------------------------------------------------------------------------------
  
  ; "Incoming string: " + SerialString.s
  
  ;------------------------------------------------------------------------------------------------
  ; Check if we have some data to encode and it is big enough to work with
  ;------------------------------------------------------------------------------------------------
  
  NormalStringLength.i     = Len(SerialString.s)
  
  ; "Incoming string length: " + Str(NormalStringLength.i)
  
  ;------------------------------------------------------------------------------------------------
  ; Is there a string and greater than 0?
  ;------------------------------------------------------------------------------------------------
  
  If NormalStringLength.i And NormalStringLength.i > 0
    
    ; "We got a string and it's longer than 0 bytes"
    
    ;----------------------------------------------------------------------------------------------
    ; Create a work area for the string to be decoded
    ;----------------------------------------------------------------------------------------------
    
    *NormalStringBuffer = AllocateMemory(StringByteLength(SerialString.s) + SizeOf(Character))
    
    ;*NormalStringBuffer   = AllocateMemory(NormalStringLength.i)
    
    ; "Normal string buffer handle: " + Str(*NormalStringBuffer)
    
    ;----------------------------------------------------------------------------------------------
    ; Check to see if the decode work area could be created
    ;----------------------------------------------------------------------------------------------
    
    If *NormalStringBuffer
      
      ; "We got a normal string buffer handle"
      
      ;--------------------------------------------------------------------------------------------
      ; Poke the serial stream into the decode work area
      ;--------------------------------------------------------------------------------------------
      
      ActualDataPoked.i  = PokeS(*NormalStringBuffer, SerialString.s, NormalStringLength.i)
      
      ; "Number of bytes that were poked into the string buffer: " + Str(ActualDataPoked.i)
      
      ;--------------------------------------------------------------------------------------------
      ; Check to see if the data made it into the work area
      ;--------------------------------------------------------------------------------------------
      
      If ActualDataPoked.i
        
        ; "Data was poked into the buffer so we can move on"
        
        ;------------------------------------------------------------------------------------------
        ; Make sure there is enough room in the work area to proceed
        ;------------------------------------------------------------------------------------------
        
        EncodedStringWorkSize.i   = NormalStringLength.i * 1.8
        
        ; "We allocated a bigger work area for the string to be encoded"
        
        ;------------------------------------------------------------------------------------------
        ; Create a work area for the encoded string
        ;------------------------------------------------------------------------------------------
        
        *EncodedStringBuffer  = AllocateMemory(EncodedStringWorkSize.i)
        
        ; "Allocated an encoding string buffer handle: " + Str(*EncodedStringBuffer)
        
        ;------------------------------------------------------------------------------------------
        ; Create a work area for the encoded string
        ;------------------------------------------------------------------------------------------
        
        If *EncodedStringBuffer
          
          ; "We managed to get a handle to the encoding string buffer so we can move on"
          
          ;----------------------------------------------------------------------------------------
          ; Encode the string and check to see what was written
          ;----------------------------------------------------------------------------------------
          
          ActualBytesEncoded.i   = Base64Encoder(*NormalStringBuffer, NormalStringLength.i, *EncodedStringBuffer, EncodedStringWorkSize.i)
          
          ; "We encoded: " + Str(ActualBytesEncoded.i) + " bytes in the encoded string buffer"
          
          ;----------------------------------------------------------------------------------------
          ; If we had data encode, now return it to the calling procedure
          ;----------------------------------------------------------------------------------------
          
          If ActualBytesEncoded.i
            
            ; "Bytes were successfully encoded so we return the encoded string"
            
            EncodedString.s = PeekS(*EncodedStringBuffer, ActualBytesEncoded.i)
            
          Else
            
            ; "No bytes of data were found encoded in the encoded string buffer"
            
          EndIf
          
          ;----------------------------------------------------------------------------------------
          ; 
          ;----------------------------------------------------------------------------------------
          
          If *NormalStringBuffer
            
            FreeMemory(*NormalStringBuffer)
            
          EndIf
          
          ;----------------------------------------------------------------------------------------
          ; 
          ;----------------------------------------------------------------------------------------
          
          If *EncodedStringBuffer
            
            FreeMemory(*EncodedStringBuffer)
            
          EndIf
          
          ;----------------------------------------------------------------------------------------
          ; 
          ;----------------------------------------------------------------------------------------
          
        Else
          
          ; "No buffer allocated to the incoming serial stream!"
          
        EndIf
        
        ;------------------------------------------------------------------------------------------
        ; 
        ;------------------------------------------------------------------------------------------
      Else
        
        ; "No bytes were poked into the work buffer"
        
      EndIf
      
      ;--------------------------------------------------------------------------------------------
      ; 
      ;--------------------------------------------------------------------------------------------
      
    Else
      
      ; "The normal string buffer could not be created"
      
    EndIf
    
    ;----------------------------------------------------------------------------------------------
    ; 
    ;----------------------------------------------------------------------------------------------
    
  Else
    
    ; "There is no incoming string to process"
    
  EndIf
  
  ;------------------------------------------------------------------------------------------------
  ; 
  ;------------------------------------------------------------------------------------------------
  
  ProcedureReturn EncodedString.s
  
  ;------------------------------------------------------------------------------------------------
  ; 
  ;------------------------------------------------------------------------------------------------
  
EndProcedure



; IDE Options = PureBasic 5.62 (Windows - x86)
; CursorPosition = 165
; FirstLine = 139
; Folding = -
; EnableXP
; CompileSourceDirectory
; EnableCompileCount = 0
; EnableBuildCount = 0