;------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
; Try to open the system database and create missing tables
;------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

Procedure OpenSystemDatabase()
  
  ;------------------------------------------------------------------------------------------------
  ; 
  ;------------------------------------------------------------------------------------------------
  
  FileHandle.i = OpenFile(#PB_Any, Program\DatabaseName)
  
  ;------------------------------------------------------------------------------------------------
  ; 
  ;------------------------------------------------------------------------------------------------
  
  If FileHandle.i
    
    ;----------------------------------------------------------------------------------------------
    ; 
    ;----------------------------------------------------------------------------------------------
    
    CloseFile(FileHandle.i)
    
    ;----------------------------------------------------------------------------------------------
    ; 
    ;----------------------------------------------------------------------------------------------
    
    Program\DatabaseHandle = OpenDatabase(#PB_Any, Program\DatabaseName, #Empty$, #Empty$)
    
    ;----------------------------------------------------------------------------------------------
    ; 
    ;----------------------------------------------------------------------------------------------
    
    If Program\DatabaseHandle
      
      ;--------------------------------------------------------------------------------------------
      ; 
      ;--------------------------------------------------------------------------------------------
      
      VacuumUpdate.s = "PRAGMA auto_vacuum = on"
      
      ;--------------------------------------------------------------------------------------------
      ; Turn on auto database vacuum
      ;--------------------------------------------------------------------------------------------
      
      If DatabaseUpdate(Program\DatabaseHandle, VacuumUpdate.s) = #DatabaseUpdateFail
        
        MessageRequester("Vacuum setup error", "Could not setup auto database vacuum.")
        
        ProcedureReturn
        
      EndIf
      
      ;--------------------------------------------------------------------------------------------
      ; Write the update needed table out
      ;--------------------------------------------------------------------------------------------
      
      TableupdateCreate.s                         + 

      "CREATE TABLE IF NOT EXISTS Tableupdate("   + 
      
      "Category TEXT, "                           + 
      
      "Recordid INTEGER PRIMARY KEY AUTOINCREMENT)"
      
      If DatabaseUpdate(Program\DatabaseHandle, TableupdateCreate.s)  <> #DatabaseUpdateFail
        
        If CountRecords("Recordid", "Tableupdate") = "0"
          
          TableInsert.s              + 

          "INSERT INTO Tableupdate(" + 
          
          "Category) "               + 
          
          "VALUES('0')"
          
          If DatabaseUpdate(Program\DatabaseHandle, TableInsert.s) <> #DatabaseUpdateFail
            
            ; Database table update worked
            
          Else
            
            MessageRequester("Table insert error", "Could not add data to Tableupdate table in database.")
            
          EndIf
          
        EndIf
        
        ; The database table already exists or was not created for some reason
        
      EndIf
      
      ;--------------------------------------------------------------------------------------------
      ; Write the appointments table out
      ;--------------------------------------------------------------------------------------------
      
      VersionCreate.s                         + 

      "CREATE TABLE IF NOT EXISTS Version("   + 
      
      "ProgramName TEXT, "                    + 
      
      "VersionNumber TEXT, "                  + 
      
      "CreatedOn TEXT, "                      + 
      
      "LastUpdate TEXT, "                     + 
      
      "Recordid INTEGER PRIMARY KEY AUTOINCREMENT)"
      
      If DatabaseUpdate(Program\DatabaseHandle, VersionCreate.s) <>  #DatabaseUpdateFail
        
        If CountRecords("Recordid", "Version") = "0"
          
          VersionInsert.s                   + 

          "INSERT INTO Version("            + 
          
          "ProgramName, "                   + 
          
          "VersionNumber, "                 + 
          
          "CreatedOn, "                     + 
          
          "LastUpdate) "                    + 
          
          "VALUES("                         + 
          
          "'" + Trim(#Basename)             + 

          "', "                             + 

          "'" + Trim(#Version)              + 

          "', "                             + 

          "'"                               + 

          FormatDate("%yyyy%mm%dd", Date()) + 

          "', "                             +

          "'"                               + 

          FormatDate("%yyyy%mm%dd", Date()) + 

          "')"
          
          If DatabaseUpdate(Program\DatabaseHandle, VersionInsert.s) = #DatabaseUpdateFail
            
            MessageRequester("Table insert error", "Could not add data to version table in database.")
            
          EndIf
          
        EndIf
        
      Else
        
        ; The database table already exists or was not created for some reason
        
      EndIf
      
      ;--------------------------------------------------------------------------------------------
      ; Write the MyInfo table out
      ;--------------------------------------------------------------------------------------------
      
      MyInfoCreate.s                        + 

      "CREATE TABLE IF NOT EXISTS MyInfo("  + 

      "Title TEXT, "                        + 

      "Information TEXT, "                  + 

      "Category TEXT, "                     + 

      "Archived TEXT, "                     + 

      "Favourite TEXT, "                    + 

      "Locked TEXT, "                       + 

      "Deleted TEXT, "                      + 

      "Updated TEXT, "                      + 

      "Owner TEXT, "                        + 

      "Attachments TEXT, "                  + 

      "Recordid INTEGER PRIMARY KEY AUTOINCREMENT)"
      
      If DatabaseUpdate(Program\DatabaseHandle, MyInfoCreate.s) <> #DatabaseUpdateFail
        
      Else
        
        MessageRequester("Table creation error", "Could not create MyInfo table in database.")
        
        ProcedureReturn
        
      EndIf
      
      ;--------------------------------------------------------------------------------------------
      ; Write the Archive table out
      ;--------------------------------------------------------------------------------------------
      
      ArchiveCreate.s                       + 

      "CREATE TABLE IF NOT EXISTS Archive(" + 

      "Title TEXT, "                        + 

      "Information TEXT, "                  + 

      "Category TEXT, "                     + 

      "Archived TEXT, "                     + 

      "Favourite TEXT, "                    + 

      "Locked TEXT, "                       + 

      "Deleted TEXT, "                      + 

      "Updated TEXT, "                      + 

      "Owner TEXT, "                        + 

      "Attachments TEXT, "                  + 

      "Recordid INTEGER PRIMARY KEY AUTOINCREMENT)"
      
      If DatabaseUpdate(Program\DatabaseHandle, ArchiveCreate.s) <> #DatabaseUpdateFail
        
      Else
        
        MessageRequester("Table creation error", "Could not create MyInfo table in database.")
        
        ProcedureReturn
        
      EndIf
      
      ;--------------------------------------------------------------------------------------------
      ; 
      ;--------------------------------------------------------------------------------------------
      
      IndexCreate.s = "CREATE INDEX IF NOT EXISTS MyInfoindex ON MyInfo(Title)"

      If DatabaseUpdate(Program\DatabaseHandle, IndexCreate.s) <> #DatabaseUpdateFail
        
      Else
        
        ; The database table already exists or was not created for some reason
        
        MessageRequester("Table creation error", "Could not create MyInfo index in database.")
        
        ProcedureReturn
        
      EndIf
      
      ;--------------------------------------------------------------------------------------------
      ; Write the Categories table out
      ;--------------------------------------------------------------------------------------------
      
      CategoryCreate.s                              + 

      "CREATE TABLE IF NOT EXISTS Categories("      + 

      "Category TEXT, "                             + 

      "Itemcount TEXT, "                            + 

      "Iconname TEXT, "                             + 

      "Recordid INTEGER PRIMARY KEY AUTOINCREMENT, "  + 

      "UNIQUE(Category))"
      
      If DatabaseUpdate(Program\DatabaseHandle, CategoryCreate.s) <> #DatabaseUpdateFail
        
      Else
        
        MessageRequester("Table creation error", "Could not create Categories table in database.")
        
        ProcedureReturn
        
      EndIf
      
      ;--------------------------------------------------------------------------------------------
      ; Write the Owners table out
      ;--------------------------------------------------------------------------------------------
      
      OwnerCreate.s                                 + 

      "CREATE TABLE IF NOT EXISTS Owners("          + 

      "Owner TEXT, "                                + 

      "Itemcount TEXT, "                            + 

      "Iconname TEXT, "                             + 

      "Recordid INTEGER PRIMARY KEY AUTOINCREMENT, "  + 

      "UNIQUE(Owner))"
      
      If DatabaseUpdate(Program\DatabaseHandle, OwnerCreate.s)  <> #DatabaseUpdateFail
        
      Else
        
        MessageRequester("Table creation error", "Could not create Owners table in database.")
        
        ProcedureReturn
        
      EndIf
      
      ;--------------------------------------------------------------------------------------------
      ; Home number, work number, email address, home page, fax number, telex number, pager number
      ;--------------------------------------------------------------------------------------------
      
      AttachmentsCreate.s                                 + 

      "CREATE TABLE IF NOT EXISTS Attachments("           + 

      "Attachmentid INTEGER, "                            + ; The Id of the record in MyInfo that this belongs to

      "Attachment BLOB, "                                 + ; Actual record number of the attachment

      "Filename TEXT, "                                   + 

      "Recordid INTEGER PRIMARY KEY AUTOINCREMENT, "      + 

      "FOREIGN KEY(Attachmentid) REFERENCES MyInfo(Recordid))"
      
      If DatabaseUpdate(Program\DatabaseHandle, AttachmentsCreate.s) <> #DatabaseUpdateFail
        
      Else
        
        ; The database table already exists or was not created for some reason
        
      EndIf

      ;--------------------------------------------------------------------------------------------
      ; 
      ;--------------------------------------------------------------------------------------------
      
      PreferencesCreate.s                         + 

      "CREATE TABLE IF NOT EXISTS Preferences("   + 

      "MD5FingerPrint TEXT, "                     +   ; MD5FingerPrint

      "FormFont TEXT, "                           +   ; FormFont
      
      "SearchLimit TEXT, "                        +   ; SearchLimit

      "LastSearch TEXT, "                         +   ; LastSearch

      "AsearchType TEXT, "                        +   ; Advanced window search type

      "AsearchField TEXT, "                       +   ; Advanced window search field

      "AsearchText TEXT, "                        +   ; Advanced window search text

      "AsortField TEXT, "                         +   ; Advanced window sort field

      "AsortDirection TEXT, "                     +   ; Advanced window sort direction

      "AlimitSearch TEXT, "                       +   ; Advanced window limit search

      "AreturnSize TEXT, "                        +   ; Advanced window limit amount

      "AcloseMode TEXT, "                         +   ; Advanced window close mode

      "AsearchMode TEXT, "                        +   ; Advanced window search mode
      
      "BalloonTooltips TEXT, "                    +   ; BalloonTooltips

      "BalloonColour TEXT, "                      +   ; BalloonColours

      "ColourBand TEXT, "                         +   ; ColourBand

      "HideDetail TEXT, "                         +   ; HideDetail

      "MessageSave TEXT, "                        +   ; MessageSave
      
      "ShowToolbar TEXT, "                        +   ; ShowToolbar

      "ScreenSnap TEXT, "                         +   ; ScreenSnap

      "StickyWindow TEXT, "                       +   ; StickyWindow

      "TitleClock TEXT, "                         +   ; TitleClock

      "Recordid INTEGER PRIMARY KEY AUTOINCREMENT)"
      
      If DatabaseUpdate(Program\DatabaseHandle, PreferencesCreate.s) <> #DatabaseUpdateFail
        
        If CountRecords("Recordid", "Preferences") = "0"
          
          PreferencesInsert.s         + 

          "INSERT INTO Preferences("  + 

          "MD5FingerPrint, "          +   ; MD5FingerPrint

          "FormFont, "                +   ; FormFont
          
          "SearchLimit, "             +   ; SearchLimit

          "LastSearch, "              +   ; LastSearch

          "AsearchType, "             +   ; Advanced window search type

          "AsearchField, "            +   ; Advanced window search field

          "AsearchText, "             +   ; Advanced window search text

          "AsortField, "              +   ; Advanced window sort field

          "AsortDirection, "          +   ; Advanced window sort direction

          "AlimitSearch, "            +   ; Advanced window limit search

          "AreturnSize, "             +   ; Advanced window limit amount

          "AcloseMode, "              +   ; Advanced window close mode

          "AsearchMode, "             +   ; Advanced window search mode
          
          "BalloonTooltips, "         +   ; BalloonTooltips

          "BalloonColour, "           +   ; BalloonColour

          "ColourBand, "              +   ; ColourBand

          "HideDetail, "              +   ; HideDetail

          "MessageSave, "             +   ; MessageSave
          
          "ShowToolbar, "             +   ; ShowToolbar

          "ScreenSnap, "              +   ; ScreenSnap

          "StickyWindow, "            +   ; StickyWindow

          "TitleClock) "              +   ; TitleClock

          "VALUES("                   +   ; 

          "'', "                      +   ; MD5FingerPrint

          "'Comic SANS MS', "         +   ; FormFont

          "'28', "                    +   ; SearchLimit

          "'', "                      +   ; LastSearch

          "'0', "                     +   ; Advanced window search type

          "'0', "                     +   ; Advanced window search field

          "'', "                      +   ; Advanced window search text

          "'0', "                     +   ; Advanced window sort field

          "'0', "                     +   ; Advanced window sort direction

          "'0', "                     +   ; Advanced window limit search

          "'25', "                    +   ; Advanced window limit amount

          "'0', "                     +   ; Advanced window close mode

          "'0', "                     +   ; Advanced window search mode

          "'1', "                     +   ; BalloonTooltips

          "'1', "                     +   ; BalloonColour

          "'1', "                     + ;   ColourBand

          "'1', "                     + ;   HideDetail

          "'1', "                     + ;   MessageSave

          "'1', "                     + ;   ShowToolbar

          "'1', "                     + ;   ScreenSnap

          "'1', "                     + ;   StickyWindow

          "'1')"                        ;   TitleClock

          If DatabaseUpdate(Program\DatabaseHandle, PreferencesInsert.s) <> #DatabaseUpdateFail
            
          Else
            
            ; The database table already exists or was not created for some reason
            
          EndIf
          
        EndIf
        
      Else
        
        MessageRequesterEBS("Table creation error", "Could not create preferences table in database.", #MB_ICONINFORMATION | #PB_MessageRequester_Ok)
        
        ; MessageRequester("Table creation error", "Could not create preferences table in database.")
        
      EndIf
      
      ;--------------------------------------------------------------------------------------------
      ; 
      ;--------------------------------------------------------------------------------------------
      
    Else
      
      MessageRequesterEBS("Database open error", "There was a serious problem attempting to connect to the system database. Could be corrupt or open by some other process. Exit MyInfo and try again.", #MB_ICONINFORMATION | #PB_MessageRequester_Ok)
      
      ;MessageRequester("Database open error", "There was a serious problem attempting to connect to the system database. Could be corrupt or open by some other process. Exit Hive and try again.", #PB_MessageRequester_Ok)
      
      ProcedureReturn
      
    EndIf 
    
    ;----------------------------------------------------------------------------------------------
    ; 
    ;----------------------------------------------------------------------------------------------
    
  Else
    
    MessageRequesterEBS("Database file open error", "Could not open or create raw database file.", #MB_ICONINFORMATION | #PB_MessageRequester_Ok)
    
    ; MessageRequester("Database file open error", "Could not open or create raw database file.", #PB_MessageRequester_Ok)
    
    ProcedureReturn
    
  EndIf
  
  ;------------------------------------------------------------------------------------------------
  ; 
  ;------------------------------------------------------------------------------------------------
  
EndProcedure


; IDE Options = PureBasic 5.62 (Windows - x86)
; CursorPosition = 456
; FirstLine = 464
; Folding = -
; EnableXP
; HideErrorLog
; CompileSourceDirectory
; EnableCompileCount = 0
; EnableBuildCount = 0
; EnableExeConstant