;------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
; 
;------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

Procedure AddNewData()
  
  ;------------------------------------------------------------------------------------------------
  ; Check if a previous window is open
  ;------------------------------------------------------------------------------------------------
  
  If Program\WindowMutex <> #True
    
    ;----------------------------------------------------------------------------------------------
    ; Try to open the data window
    ;----------------------------------------------------------------------------------------------
    
    If Window_Data()
      
      ;--------------------------------------------------------------------------------------------
      ; Prevent user input on the main window
      ;--------------------------------------------------------------------------------------------
      
      DisableWindow(#Window_MyInfo, #True)
      
      ;--------------------------------------------------------------------------------------------
      ; Warn other components a window is open and not to attempt to open another
      ;--------------------------------------------------------------------------------------------
      
      Program\WindowMutex = #True
      
      ;--------------------------------------------------------------------------------------------
      ; Add keyboard shortcuts as backup to the buttons, Good for people with sore mouse wrists
      ;--------------------------------------------------------------------------------------------
      
      AddKeyboardShortcut(#Window_Data, #ShortcutValue_SaveData,         #KeyboardShortcut_SaveData)
      
      AddKeyboardShortcut(#Window_Data, #ShortcutValue_ExportData,       #KeyboardShortcut_ExportData)
      
      AddKeyboardShortcut(#Window_Data, #ShortcutValue_ImportData,       #KeyboardShortcut_ImportData)
      
      AddKeyboardShortcut(#Window_Data, #ShortcutValue_HelpData,         #KeyboardShortcut_HelpData)
      
      AddKeyboardShortcut(#Window_Data, #ShortcutValue_ExitData,         #KeyboardShortcut_ExitData)
      
      AddKeyboardShortcut(#Window_Data, #ShortcutValue_EscapeData,       #KeyboardShortcut_EscapeData)
      
      AddKeyboardShortcut(#Window_Data, #ShortcutValue_SearchData,       #KeyboardShortcut_SearchData)
      
      ;--------------------------------------------------------------------------------------------
      ; Add the hover enabled images
      ;--------------------------------------------------------------------------------------------
      
      AddHoverGadget(GadgetID(#Gadget_Data_SaveData))
      
      AddHoverGadget(GadgetID(#Gadget_Data_ExportData))
      
      AddHoverGadget(GadgetID(#Gadget_Data_ImportData))
      
      AddHoverGadget(GadgetID(#Gadget_Data_SearchData))
      
      AddHoverGadget(GadgetID(#Gadget_Data_HelpData))
      
      AddHoverGadget(GadgetID(#Gadget_Data_ExitData))
      
      ;--------------------------------------------------------------------------------------------
      ; Balloon tooltips with keyboard shortcut display
      ;--------------------------------------------------------------------------------------------
      
      If Program\BalloonFontId
        
        BalloonToolTip(#Window_Data, #Gadget_Data_SaveData,   Program\BalloonFontId, GetBalloonColour(), ShortTooltip\SaveData,    #CRLF$ + GetShortCutText(#ShortcutValue_SaveData)   + #CRLF$ + #CRLF$ + WordWrapString(LongTooltip\SaveData,    30),  GetIconHandle(#Image_Data_SaveData))
        
        BalloonToolTip(#Window_Data, #Gadget_Data_ExportData, Program\BalloonFontId, GetBalloonColour(), ShortTooltip\ExportData,  #CRLF$ + GetShortCutText(#ShortcutValue_ExportData) + #CRLF$ + #CRLF$ + WordWrapString(LongTooltip\ExportData,  30),  GetIconHandle(#Image_Data_ExportData))
        
        BalloonToolTip(#Window_Data, #Gadget_Data_ImportData, Program\BalloonFontId, GetBalloonColour(), ShortTooltip\ImportData,  #CRLF$ + GetShortCutText(#ShortcutValue_ImportData) + #CRLF$ + #CRLF$ + WordWrapString(LongTooltip\ImportData,  30),  GetIconHandle(#Image_Data_ImportData))
        
        BalloonToolTip(#Window_Data, #Gadget_Data_SearchData, Program\BalloonFontId, GetBalloonColour(), ShortTooltip\SearchData,  #CRLF$ + GetShortCutText(#ShortcutValue_SearchData) + #CRLF$ + #CRLF$ + WordWrapString(LongTooltip\SearchData,  30),  GetIconHandle(#Image_Data_SearchData))
        
        BalloonToolTip(#Window_Data, #Gadget_Data_HelpData,   Program\BalloonFontId, GetBalloonColour(), ShortTooltip\HelpData,    #CRLF$ + GetShortCutText(#ShortcutValue_HelpData)   + #CRLF$ + #CRLF$ + WordWrapString(LongTooltip\HelpData,    30),  GetIconHandle(#Image_Data_HelpData))
        
        BalloonToolTip(#Window_Data, #Gadget_Data_ExitData,   Program\BalloonFontId, GetBalloonColour(), ShortTooltip\ExitData,    #CRLF$ + GetShortCutText(#ShortcutValue_ExitData)   + #CRLF$ + #CRLF$ + WordWrapString(LongTooltip\ExitData,    30),  GetIconHandle(#Image_Data_ExitData))
        
      Else
        
        MessageRequesterEBS("Balloon font load error", "Could not load the ballon font so no tooltips will be loaded", #MB_ICONINFORMATION | #PB_MessageRequester_Ok)
        
        ;MessageRequester("Balloon font load error", "Could not load the ballon font so no tooltips will be loaded", #PB_MessageRequester_Ok)
        
      EndIf
      
      ;--------------------------------------------------------------------------------------------
      ; Remove theme support from the comment box or it retains a border even when API removed
      ;--------------------------------------------------------------------------------------------
      
      RemoveBorders(#Gadget_Data_Details)
      
      ;--------------------------------------------------------------------------------------------
      ; Get the last record number in the database and give the next probable record number
      ;--------------------------------------------------------------------------------------------
      
      NextRecord.i = Val(LastRecord("MyInfo", "Recordid")) + 1
      
      ;--------------------------------------------------------------------------------------------
      ; Use the common message as a window identifier
      ;--------------------------------------------------------------------------------------------
      
      Program\LastWindow = "Add New MyInfo Item"
      
      ;--------------------------------------------------------------------------------------------
      ; Set the window title with the common message and next record number
      ;--------------------------------------------------------------------------------------------
      
      SetWindowTitle(#Window_Data, Program\LastWindow + "   --   Record number: " + Str(NextRecord.i) + " ?")
      
      ;--------------------------------------------------------------------------------------------
      ; 
      ;--------------------------------------------------------------------------------------------
      
      FillComboBoxImage(#Gadget_Data_cCategory, "SELECT Category, Iconname FROM Categories ORDER BY Category", Program\CategoryIcons, #Image_ListNocategory)
      
      ;--------------------------------------------------------------------------------------------
      ; 
      ;--------------------------------------------------------------------------------------------
      
      FillComboBoxImage(#Gadget_Data_Owner, "SELECT Owner, Iconname FROM Owners ORDER BY Owner", Program\OwnerIcons, #Image_ListNoowner)
      
      ;--------------------------------------------------------------------------------------------
      ; Set the current operation details on the the main window status bar
      ;--------------------------------------------------------------------------------------------
      
      SetGadgetText(#Gadget_Data_Recordnumber,  Str(NextRecord.i) + " ?")
      
      ;--------------------------------------------------------------------------------------------
      ; Disable the favourite icon in a new record as it isn't set here
      ;--------------------------------------------------------------------------------------------
      
      HideGadget(#Gadget_Data_Status, #True)
      
      ;--------------------------------------------------------------------------------------------
      ; Set the current operation details on the the main window status bar
      ;--------------------------------------------------------------------------------------------
      
      SetInfoBarArea("Headings",  "Info",  Program\LastWindow + "   --   Record number: " + Str(NextRecord.i) + " ?", #PB_Compiler_Procedure)
      
      ;--------------------------------------------------------------------------------------------
      ; Set focus on the appointment name field
      ;--------------------------------------------------------------------------------------------
      
      SetActiveGadget(#Gadget_Data_Title)
      
      ;--------------------------------------------------------------------------------------------
      ; 
      ;--------------------------------------------------------------------------------------------
      
      ;HideWindow(#Window_Data, #False)
      
      ;--------------------------------------------------------------------------------------------
      ; 
      ;--------------------------------------------------------------------------------------------
      
    Else
      
      SetInfoBarArea("Headings",  "Error",  "Could not open the [" + Program\LastWindow + "] window.", #PB_Compiler_Procedure)
      
    EndIf
    
    ;----------------------------------------------------------------------------------------------
    ; 
    ;----------------------------------------------------------------------------------------------
    
  Else
    
    SetInfoBarArea("Headings",  "Warn",  "The [" +  Program\LastWindow + "] window is still open, close that one first.", #PB_Compiler_Procedure)
    
  EndIf
  
  ;------------------------------------------------------------------------------------------------
  ; 
  ;------------------------------------------------------------------------------------------------
  
EndProcedure


; IDE Options = PureBasic 5.62 (Windows - x86)
; CursorPosition = 82
; FirstLine = 68
; Folding = -
; EnableXP
; CompileSourceDirectory
; EnableCompileCount = 0
; EnableBuildCount = 0
; EnableExeConstant