;------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
; Search MyInfo's database with advanced SQL tools and limits
;------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

Procedure AdvancedSearch()
  
  ;------------------------------------------------------------------------------------------------
  ; Check if an existing window is opened
  ;------------------------------------------------------------------------------------------------
  
  If Program\WindowMutex <> #True
    
    ;----------------------------------------------------------------------------------------------
    ; Try to open the data form window
    ;----------------------------------------------------------------------------------------------
    
    If Window_Search()
      
      ;--------------------------------------------------------------------------------------------
      ; Window was opened so set mutex to prevent other windows from opening
      ;--------------------------------------------------------------------------------------------
      
      Program\WindowMutex = #True
      
      ;--------------------------------------------------------------------------------------------
      ; Prevent user input on the main window
      ;--------------------------------------------------------------------------------------------
      
      DisableWindow(#Window_MyInfo, #True)
      
      ;--------------------------------------------------------------------------------------------
      ; 
      ;--------------------------------------------------------------------------------------------
      
      StickyWindow(#Window_Search, #True)
      
      ;--------------------------------------------------------------------------------------------
      ; 
      ;--------------------------------------------------------------------------------------------
      
      SetWindowTheme_(GadgetID(#Gadget_Search_oSortAcending),       @null.w, @null.w)
      
      SetWindowTheme_(GadgetID(#Gadget_Search_oSortDescending),     @null.w, @null.w)
      
      SetWindowTheme_(GadgetID(#Gadget_Search_cbQueryReturnLimit),  @null.w, @null.w)
      
      SetWindowTheme_(GadgetID(#Gadget_Search_sQueryReturnLimit),   @null.w, @null.w)
      
      SetWindowTheme_(GadgetID(#Gadget_Search_cbCheckEnter),        @null.w, @null.w)
      
      SetWindowTheme_(GadgetID(#Gadget_Search_oCheckOr),            @null.w, @null.w)
      
      SetWindowTheme_(GadgetID(#Gadget_Search_oCheckAnd),           @null.w, @null.w)
      
      ;--------------------------------------------------------------------------------------------
      ; 
      ;--------------------------------------------------------------------------------------------
      
      AddKeyboardShortcut(#Window_Search, #ShortcutValue_RunSearch,      #KeyboardShortCut_RunSearch)
      
      AddKeyboardShortcut(#Window_Search, #ShortcutValue_ReturnSearch,   #KeyboardShortcut_ReturnSearch)
      
      AddKeyboardShortcut(#Window_Search, #ShortcutValue_HelpSearch,     #KeyboardShortcut_HelpSearch)
      
      AddKeyboardShortcut(#Window_Search, #ShortcutValue_ExitSearch,     #KeyboardShortcut_ExitSearch)
      
      AddKeyboardShortcut(#Window_Search, #ShortcutValue_EscapeSearch,   #KeyboardShortcut_EscapeSearch)
      
      ;--------------------------------------------------------------------------------------------
      ; Setup the form hover controls
      ;--------------------------------------------------------------------------------------------
      
      AddHoverGadget(GadgetID(#Gadget_Search_RunSearch))
      
      AddHoverGadget(GadgetID(#Gadget_Search_HelpSearch))
      
      AddHoverGadget(GadgetID(#Gadget_Search_ExitSearch))
      
      ;--------------------------------------------------------------------------------------------
      ; Balloon tooltips with keyboard shortcut display
      ;--------------------------------------------------------------------------------------------
      
      If Program\BalloonFontId
        
        BalloonToolTip(#Window_Search, #Gadget_Search_RunSearch,  Program\BalloonFontId, GetBalloonColour(), ShortTooltip\RunSearch,   #CRLF$ + GetShortCutText(#ShortcutValue_RunSearch)  + #CRLF$ + #CRLF$ + WordWrapString(LongTooltip\RunSearch,   30),  GetIconHandle(#Image_MyInfo_Add))
        
        BalloonToolTip(#Window_Search, #Gadget_Search_HelpSearch, Program\BalloonFontId, GetBalloonColour(), ShortTooltip\HelpSearch,  #CRLF$ + GetShortCutText(#ShortcutValue_HelpSearch) + #CRLF$ + #CRLF$ + WordWrapString(LongTooltip\HelpSearch,  30),  GetIconHandle(#Image_MyInfo_Archive))
        
        BalloonToolTip(#Window_Search, #Gadget_Search_ExitSearch, Program\BalloonFontId, GetBalloonColour(), ShortTooltip\ExitSearch,  #CRLF$ + GetShortCutText(#ShortcutValue_ExitSearch) + #CRLF$ + #CRLF$ + WordWrapString(LongTooltip\ExitSearch,  30),  GetIconHandle(#Image_MyInfo_Attach))
        
      Else
        
        MessageRequesterEBS("Balloon font load error", "Could not load the ballon font so no tooltips will be loaded", #MB_ICONINFORMATION | #PB_MessageRequester_Ok)
        
        ;MessageRequester("Balloon font load error", "Could not load the ballon font so no tooltips will be loaded", #PB_MessageRequester_Ok)
        
      EndIf
      
      ;--------------------------------------------------------------------------------------------
      ; Set the last window message for the closure routine later
      ;--------------------------------------------------------------------------------------------
      
      Program\LastWindow = "Search passwords, advanced mode."
      
      ;--------------------------------------------------------------------------------------------
      ; Show new record number and status in the data window
      ;--------------------------------------------------------------------------------------------
      
      SetWindowTitle(#Window_Search, Program\LastWindow)
      
      ;--------------------------------------------------------------------------------------------
      ; Limit the number of search box characters and set the cue banner text. These variables are 
      ; set in the MyInfo_Myconstants file.
      ;--------------------------------------------------------------------------------------------
      
      SendMessage_(GadgetID(#Gadget_Search_SearchText), #EM_SETLIMITTEXT, 67, 0)
      
      SendMessage_(GadgetID(#Gadget_Search_SearchText), #EM_SETCUEBANNER,  1,  Cue\SearchCueText)
      
      ;--------------------------------------------------------------------------------------------
      ; Set initial search cue font
      ;--------------------------------------------------------------------------------------------
      
      SetGadgetFont(#Gadget_Search_SearchText, FontID(Cue\SearchCueFont))
    
      ;--------------------------------------------------------------------------------------------
      ; Add the Search type operators
      ;--------------------------------------------------------------------------------------------
      
      Restore BeginSearchTypes
      
      While TempString.s <> "_EndSearchTypes_"
        
        Read.s TempString.s
        
        If TempString.s <> "_EndSearchTypes_"
          
          AddGadgetItem(#Gadget_Search_SearchType, #AtTheEndOfTheList, TempString.s)
          
        EndIf
        
      Wend
      
      TempString.s = "<>"
      
      ;--------------------------------------------------------------------------------------------
      ; 
      ;--------------------------------------------------------------------------------------------
      
      If Search\SearchWindowType
        
        SetGadgetState(#Gadget_Search_SearchType,   Search\SearchWindowType)
        
      Else
        
        SetGadgetState(#Gadget_Search_SearchType,   #FirstItem)
        
      EndIf
      
      ;--------------------------------------------------------------------------------------------
      ; Add the Search type operators
      ;--------------------------------------------------------------------------------------------
      
      AddGadgetItem(#Gadget_Search_SearchField, #AtTheEndOfTheList, "All fields")
      
      Restore BeginSqlFields
      
      While TempString.s <> "_EndSqlFields_"
        
        Read.s TempString.s
        
        If TempString.s <> "_EndSqlFields_"
          
          AddGadgetItem(#Gadget_Search_SearchField, #AtTheEndOfTheList, TempString.s)
          
        EndIf
        
      Wend
      
      TempString.s = "<>"
      
      ;--------------------------------------------------------------------------------------------
      ; 
      ;--------------------------------------------------------------------------------------------
      
      If Search\SearchWindowField
        
        SetGadgetState(#Gadget_Search_SearchField,  Search\SearchWindowField)
        
      Else
        
        SetGadgetState(#Gadget_Search_SearchField,  #FirstItem)
        
      EndIf
      
      ;--------------------------------------------------------------------------------------------
      ; Set the last search text to the window
      ;--------------------------------------------------------------------------------------------
      
      SetGadgetText(#Gadget_Search_SearchText,    Search\SearchWindowText)
      
      ;--------------------------------------------------------------------------------------------
      ; 
      ;--------------------------------------------------------------------------------------------
      
      SendMessage_(GadgetID(#Gadget_Search_SearchText), #EM_SETSEL, Len(Search\SearchWindowText), Len(Search\SearchWindowText))
      
      ;--------------------------------------------------------------------------------------------
      ; Add the available sort fields
      ;--------------------------------------------------------------------------------------------
      
      Restore BeginSqlFields
      
      While TempString.s <> "_EndSqlFields_"
        
        Read.s TempString.s
        
        If TempString.s <> "_EndSqlFields_" And TempString.s <> "All fields"
          
          AddGadgetItem(#Gadget_Search_SortField, #AtTheEndOfTheList, TempString.s)
          
        EndIf
        
      Wend
      
      TempString.s = "<>"
      
      ;--------------------------------------------------------------------------------------------
      ; Set the default field to sort on
      ;--------------------------------------------------------------------------------------------
      
      If Search\SearchWindowSortField
        
        SetGadgetState(#Gadget_Search_SortField,  Search\SearchWindowSortField)
        
      Else
        
        SetGadgetState(#Gadget_Search_SortField,  #FirstItem)
        
      EndIf
      
      ;--------------------------------------------------------------------------------------------
      ; Set the default sort direction
      ;--------------------------------------------------------------------------------------------
      
      If Search\SearchWindowSortDirection = #False
        
        SetGadgetState(#Gadget_Search_oSortAcending, #True)  
        
      ElseIf Search\SearchWindowSortDirection = #True
        
        SetGadgetState(#Gadget_Search_oSortDescending, #True)
        
      EndIf
      
      ;--------------------------------------------------------------------------------------------
      ; Set the checkbox to turn on or off MyInfo number limiting during searches
      ;--------------------------------------------------------------------------------------------
      
      SetGadgetState(#Gadget_Search_cbQueryReturnLimit, Search\SearchWindowQueryReturnLimit)
      
      ;--------------------------------------------------------------------------------------------
      ; Set the number of recipes to return during a search
      ;--------------------------------------------------------------------------------------------
      
      SetGadgetState(#Gadget_Search_sQueryReturnLimit,      Search\SearchWindowQueryReturnSize)
      
      SetGadgetText(#Gadget_Search_sQueryReturnLimit,   Str(Search\SearchWindowQueryReturnSize))
      
      ;--------------------------------------------------------------------------------------------
      ; Set the close mode for the window
      ;--------------------------------------------------------------------------------------------
      
      SetGadgetState(#Gadget_Search_cbCheckEnter,       Search\SearchWindowCloseMode)
      
      ;--------------------------------------------------------------------------------------------
      ; Set the multiple search parameters if any
      ;--------------------------------------------------------------------------------------------
      
      If Search\SearchWindowMode = #False
        
        SetGadgetState(#Gadget_Search_oCheckOr, #True)
        
      ElseIf Search\SearchWindowMode = #True
        
        SetGadgetState(#Gadget_Search_oCheckAnd, #True)
        
      EndIf
      
      ;--------------------------------------------------------------------------------------------
      ; Show information in the statusbar area
      ;--------------------------------------------------------------------------------------------
      
      SetInfoBarArea("Headings", "Info", Program\LastWindow, #PB_Compiler_Procedure)
      
      ;--------------------------------------------------------------------------------------------
      ; Make sure the data form window is topmost
      ;--------------------------------------------------------------------------------------------
      
      SetActiveWindow(#Window_Search)
      
      ;--------------------------------------------------------------------------------------------
      ; Make sure the version field is selected
      ;--------------------------------------------------------------------------------------------
      
      SetActiveGadget(#Gadget_Search_SearchText)
      
      ;--------------------------------------------------------------------------------------------
      ; 
      ;--------------------------------------------------------------------------------------------
      
    Else
      
      SetInfoBarArea("Headings", "Error", "The program window could not be opened.", #PB_Compiler_Procedure)
      
    EndIf
    
    ;----------------------------------------------------------------------------------------------
    ; 
    ;----------------------------------------------------------------------------------------------
    
  Else
    
    SetInfoBarArea("Headings", "Warn", "Another program window is still open.", #PB_Compiler_Procedure)
    
  EndIf
  
  ;------------------------------------------------------------------------------------------------
  ; 
  ;------------------------------------------------------------------------------------------------
  
EndProcedure


; IDE Options = PureBasic 5.62 (Windows - x86)
; CursorPosition = 90
; FirstLine = 67
; Folding = -
; EnableXP
; HideErrorLog
; CompileSourceDirectory
; EnableCompileCount = 0
; EnableBuildCount = 0
; EnableExeConstant