;------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
; Generic routines
;------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

Declare.s FormatField(InString.s, FieldLen.i)                                                                     ; Format the passed string to the correct field length for printing/copying to clipboard etc
Declare.s FormatMenu(HeadingString.s, ShortcutString.s, FieldLen.i)                                               ; Format menu text and shortcut strings to a predefined length
Declare.s GetShortcutText(Constant.i)                                                                             ; Get the text of the associated shortcut constant
Declare.s KillQuote(Instring.s)                                                                                   ; Kill double quotes in strings for display purposes
Declare   LastLine(Gadget.i, LineNumber.i)                                                                        ; Go to the last line of a ListIconGadget
Declare.s MakeString(StringLen.i, Character.s)                                                                    ; Create a string of spaces or other characters. Pass it any number of characters and watch it grow!
Declare.s MakeSureDirectoryPathExists(Directory.s)                                                                ; Need a unicode aware version of the API directory creator
Declare   MixedComboCaseEx(Gadget.i)                                                                              ; First letter uppercase in a combo box. srod!.
Declare   MixedStringCase(Gadget.i)                                                                               ; First letter uppercase in a combo box. srod!.
Declare.s ProperCaseString(StringToBeFlipped.s)                                                                   ; Flip a mixed string into all proper case
Declare.s RepQuote(Instring.s)                                                                                    ; Uncle Berikco's routine to properly replace single quotes with double for SQL passing
Declare   RemoveBorders(GadgetConstant.i)                                                                         ; Remove borders from various gadgets
Declare.s SetSearchQuery(SearchOperator.s, SearchString.s)                                                        ; Alias the Search type text to SQLite operators
Declare   StringFirstUpper(GadgetId.i)                                                                            ; Convert the first character of a string to uppercase
Declare.s TextToXML(TextString.s)                                                                                 ; Replace bad characters with XML compatible ones. Thorsten Hoeppner
Declare   xcm(x.f)                                                                                                ; Calculate x axis movement for printer dimensions
Declare.s XMLToText(XMLString.s)                                                                                  ; Strip illegal characters from text strings
Declare   ycm(x.f)                                                                                                ; Calculate y axis movement for printer dimensions

;------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
; Custom routines
;------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

Declare   BalloonToolTip(pbWindowNumber.i, pbGadgetNumber.i, pbFontHandle.i, balloonColour.i, balloonTipTitle.s, balloonTipText.s,  pbIconHandle.i) ; 
Declare   CheckEnterGadget()                                                                                      ; Check which gadget had the focus at the time ENTER was pressed
Declare   CloseMyWindow(Window.i)                                                                                 ; Fade the window out gently, close it and reset the mutex
Declare.i GetBalloonColour()                                                                                      ; Set a random or prechosen balloon tooltip colour
Declare.i GetIconHandle(ImageConstant.i)                                                                          ; Copy the icon into memory while deleting the original image copied from
Declare.s GetPartBetweenBrackets(LineToProcess.s)                                                                 ; Extract the text between pointy brackets
Declare.s IsoDate(DateString.s, Direction.s)                                                                      ; Convert an ISO format date to Australian and back
Declare   SetInfoBarArea(FieldName.s, FieldStatus.s, MessageContent.s = "0", ModuleName.s = #Empty$)              ; Send messages and error reports to the fake status bar area
Declare   SetHoverIcons(Gadget.i, Image.i, State.s)                                                               ; Set the correct icons to the selected hoverable gadget
Declare   SortTitleStrings(HighlightString.s)                                                                     ; Sort ListIconGadget as you need it. Based on Reg Venaglia's routine.
Declare   StatusBarButtonTip(WindowId.i, Section.s)                                                               ; Sort ListIconGadget as you need it. Based on Reg Venaglia's routine.
Declare   TitleBarClock(ObjectHandle.i, Message.i, EventNumber.i, TimePeriod.i)                                   ; Run an asynchronous clock in the window title bar
Declare   WriteErrorLog(ModuleName.s = #Empty$, Message.s = #Empty$)                                              ; Write the error log to disk in the temporary directory
Declare.s WordWrapString(SourceString.s, StringWidth.i, DelimiterList.s = "- " + #TAB$, NewLineString.s = #LF$)   ; Wrap a string to a specified boundary

;------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
; Custom form callbacks
;------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

Declare   MainWindowCallback(hWnd, uMsg, wParam, lParam)                                                          ; Window snap, strikeoutfont and more

;------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
; Database functions
;------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

Declare.s CountRecords(TableField.s, TableName.s)                                                                 ; Count the number of records in an SQLite database
Declare.s DatabaseLastInsertRowId()                                                                               ; Get the ID of the last inserted record
Declare   FillCombo(Gadget.i, DatabaseQuery.s, ColumnIcon.i = 0)                                                  ; Fill a combobox gadget from a linked list, SQLite version
Declare   FillComboBoxImage(Gadget.i, DatabaseQuery.s, Storage.s, NormalIcon.i)                                   ; Fill a combobox with items and their stored images
Declare.s LastRecord(TableName.s, TableField.s)                                                                   ; Find the last record number in the database, not necessarily the last record though due to deletions (fang)
Declare   OpenSystemDatabase()                                                                                    ; Try to open the system database and create missing tables

;------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
; Export and import routines, File menu
;------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

Declare  BackupDatabase()                                                                                         ; Backup the current database to the named day of the week
Declare  ExportDatabase()                                                                                         ; Export data data to a standard XML file
Declare  ImportDatabase()                                                                                         ; Import an XML file of MyInfo entries
Declare  OpenAdatabase()                                                                                          ; Open up a database other than the default
Declare  ProcessMyInfoNode(*CurrentNode, CurrentSublevel.i)                                                       ; Decode MyInfo node data
Declare  VacuumDeletedData()                                                                                      ; Clean all deleted items from the database immediately

;------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
; Categories management, Category menu
;------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

Declare   AddCategory()                                                                                           ; Add a new category to the database
Declare   ChangeCategory()                                                                                        ; Change the category of the title that was dropped to what it was dropped on
Declare   CopyCategory()                                                                                          ; Copy the current category to the clipboard
Declare   DeleteCategory()                                                                                        ; Delete the selected category
Declare   DragToCategory()                                                                                        ; Change the category of a title by dragging and dropping it onto the category bar
Declare   EditCategory()                                                                                          ; Edit the selected category
Declare   ExportCategories()                                                                                      ; Export the list of categories to disk
Declare   ImportCategories()                                                                                      ; Import a text file list of categories from disk
Declare   LoadCategories()                                                                                        ; Load all the categories into the category bar
Declare   SaveCategory()                                                                                          ; Save the changed category data to the database
Declare   ShowCategory()                                                                                          ; Show the items that belong to a particular category
Declare   SortCategoryEntries()                                                                                   ; Sort the category list after an add or edit
Declare   UpdateCategoryItemCount(Category.s)                                                                     ; Update the category itemcount field each time we add, change or delete addresses or categories

;------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
; All data handling, Data menu
;------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

Declare   AddNewData()                                                                                            ; Add a new serially item
Declare   AttachItemToData()                                                                                      ; Attach a document or file to the current record
Declare   CopyDataToClipboard()                                                                                   ; Copy highlighted data to the system clipboard
Declare   DataFormTemplate()                                                                                      ; Standard data form template for convenience
Declare   DisplayTitleString(LineNumber.i, DisplayMode.s)                                                         ; Handle all list display jobs from add, edit, list, import modes etc
Declare   DeleteCurrentData()                                                                                     ; Mark the selected item as deleted
Declare   DragnewData()                                                                                           ; Enable importing data by dropping a text file
Declare   EditOldData()                                                                                           ; Edit the old serially data
Declare   ExportItemToDisk()                                                                                      ; Export the currently selected item to disk
Declare   ImportItemFromDisk()                                                                                    ; Import a preformatted text file from disk
Declare   KillCurrentData()                                                                                       ; KIll the currently selected data permanently
Declare   LockCurrentData()                                                                                       ; Lock the current record against deletion and editing
Declare   MarkAsFavourite()                                                                                       ; Mark this entry as a favourite item
Declare   PrintThisData()                                                                                         ; Print all or selected records
Declare   SaveThisData()                                                                                          ; Save the new or edited MyInfo data
Declare   SearchDataForm()                                                                                        ; Data form search window
Declare   SearchDataFormNow()                                                                                     ; Search within the data form itself
Declare   SearchDatabase()                                                                                        ; Search for items in the database
Declare   ShowMyInfoData()                                                                                        ; Show the data string for MyInfo on the screen
Declare   UpdateTitleAttachmentCount(CurrentLine.i, CurrentRecord.s)                                              ; Update the attachment counter in the title list
Declare   VacuumDeletedData()                                                                                     ; Clean all deleted items from the database immediately

; Scan document from TWAIN source

Declare.l AquireDocument(wPixTypes.l, dResolution.d, reset.w)                                                     ; Aquire a document image from the scanner
Declare   ScanDocument()                                                                                          ; Scan an  into the current record
Declare   ScanNextDocumentPage()                                                                                  ; Scan the next document page
Declare   SaveScannedDocumentPages()                                                                              ; Save the scanned document pages

;------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
; Utilities menu
;------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

Declare   AdvancedSearch()                                                                                        ; Open the extended SQL search window
Declare   AdvancedSearchNow()                                                                                     ; Perform the extended SQL search now
Declare   SelectAllData()                                                                                         ; Select all the data in the list for some operations

Declare   ManageOwners()                                                                                          ; Manage the owner list table
Declare   AddOwner()                                                                                              ; Add a new Owner to the Owner bar
Declare   CopyOwner()                                                                                             ; Copy the current owner to the clipboard
Declare   DeleteOwner()                                                                                           ; Delete an existing Owner from the owner bar
Declare   EditOwner()                                                                                             ; Edit the old owner in the database
Declare   SaveOwner()                                                                                             ; Save the new or changed Owner information
Declare   SortOwners()                                                                                            ; Sort ListIconGadget as you need it. Based on Reg Venaglia's routine.
Declare   SyncOwnerlist()                                                                                         ; Update category item count in owner list and create the list of owners list()                                                                                    ; Fix the ItemCount field in the owner table on database changes
Declare   UpdateOwnerItemCount(OwnerName.s)                                                                       ; Update the Owner itemcount field each time we add, change or delete owners

;------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
; Program toggles, Options menu  
;------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

Declare   ToggleBalloonToolTips()                                                                                 ; Turn balloon tooltips on or off
Declare   ToggleBalloonColour()                                                                                   ; Toggle random or prechosen balloon tooltip colour
Declare   ToggleColourBand()                                                                                      ; Turn alternate row colour banding on and off
Declare   ToggleHideDetail()                                                                                      ; Turn on or off the showing of the detail pane
Declare   ToggleMessageSave()                                                                                     ; Turn on or off messages after saving
Declare   ToggleWindowToolBars()                                                                                  ; Turn on or off the showing of the main toolbar
Declare   ToggleScreenSnap()                                                                                      ; Turn snapping to screen borders on and off
Declare   ToggleStickyWindow()                                                                                    ; Set this window on top of others or alarm window will not show
Declare   ToggleTitleBarClock()                                                                                   ; Turn time updating on in the title bar

;------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
; Program help, Help menu
;------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

Declare   AboutMyInfo()                                                                                           ; A quick 'about me' blurb
Declare   KeysboardShortcuts()                                                                                    ; Show a list of all the keyboard shortcuts
Declare   ViewErrorLog()                                                                                          ; View the system error log file

;------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
; Help modules
;------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

Declare   EditHelp()                                                                                              ; Change the current read only state of the current help page
Declare   LoadHelp()                                                                                              ; Load an Rtf text file into the help area
Declare   SaveHelp()                                                                                              ; Save the current help contents to an Rtf text file on disk
Declare   SearchHelp()                                                                                            ; Search the MyInfo help file and highlight text. RASHAD
Declare   ShowHelp(Section.s)                                                                                     ; Adaptable help file that loads RTF from datasection
Declare   PrintHelp()                                                                                             ; Print the contents of the current help page

;------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
; All security sections here
;------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

Declare   NewPassWordWindow()                                                                                     ; Open the new password window
Declare   CheckNewPassWord()                                                                                      ; Check that there is a new password

Declare   StartPassWordWindow()                                                                                   ; Show a password lock screen at program start
Declare   CheckStartUpPassWord()                                                                                  ; Check the startup unlocking password

Declare   EditpassWordWindow()                                                                                    ; Open the edit password window
Declare   CheckpassWords()                                                                                        ; Check that the old password and confirmation passwords match

;------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
; Lock and unlock the program window to the system tray
;------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

Declare   LockProgramWindow()                                                                                     ; Lock the program window now
Declare   UnLockProgramWindow()                                                                                   ; UnLock the program window now
Declare   CheckUnlockPassWord()                                                                                   ; Check the unlocking password

;------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
; Minimising and maximising the program window
;------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

Declare   HideProgramWindow()                                                                                     ; Hide the program window
Declare   ShowProgramWindow()                                                                                     ; Show the program window

;------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
; Modules required to run only once
;------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

Declare   OneTimeCategoryUpdate()                                                                                 ; Update category item count in category list
Declare   OneTimeMakeOwners()                                                                                     ; Create the new owners table on startup and insert them
Declare   OneTimeConvertRtfDatabase()                                                                             ; Convert the rtf data in the info field to normal text
; IDE Options = PureBasic 5.62 (Windows - x86)
; CursorPosition = 162
; FirstLine = 138
; EnableAsm
; EnableXP
; CompileSourceDirectory
; EnableCompileCount = 1
; EnableBuildCount = 0
; EnableExeConstant